@@ -40,7 +40,7 @@ where
     } else {
         // This is a bit of an ugly case: We have measured values, but they're
         // all zero.  If this happens, the bandwidth authorities exist but they
-        // very confused: we should fall back to uniform wrighting.
+        // very confused: we should fall back to uniform weighting.
         BandwidthFn::Uniform
     }
 }
@@ -136,12 +136,14 @@ impl std::ops::Div<u32> for RelayWeight {
 
 impl RelayWeight {
     /// Return the largest weight that we give for this kind of relay.
+    // The unwrap() is safe because array is nonempty.
+    #[allow(clippy::unwrap_used)]
     fn max_weight(&self) -> u32 {
         [self.as_guard, self.as_middle, self.as_exit, self.as_dir]
             .iter()
             .max()
             .copied()
-            .unwrap() // unwrap is safe since the input is nonempty.
+            .unwrap()
     }
     /// Return the weight we should give this kind of relay's
     /// bandwidth for a given role.
@@ -234,7 +236,7 @@ impl WeightSet {
         // we shift, to improve accuracy.  We know that this will be
         // safe, since the inputs are both u32, and so cannot overflow
         // a u64.
-        let router_weight = (router_bw as u64) * (ws.for_role(role) as u64);
+        let router_weight = u64::from(router_bw) * u64::from(ws.for_role(role));
         router_weight >> self.shift
     }
 
@@ -246,11 +248,11 @@ impl WeightSet {
         let total_bw = consensus
             .relays()
             .iter()
-            .map(|rs| bandwidth_fn.apply(rs.weight()) as u64)
+            .map(|rs| u64::from(bandwidth_fn.apply(rs.weight())))
             .sum();
         let p = consensus.bandwidth_weights();
 
-        Self::from_parts(bandwidth_fn, total_bw, weight_scale, p)
+        Self::from_parts(bandwidth_fn, total_bw, weight_scale, p).validate(consensus)
     }
 
     /// Compute the correct WeightSet given a bandwidth function, a
@@ -310,10 +312,11 @@ impl WeightSet {
 
         // This is the largest weight value.
         // The unwrap() is safe because `w` is nonempty.
+        #[allow(clippy::unwrap_used)]
         let w_max = w.iter().map(RelayWeight::max_weight).max().unwrap();
 
         // We want "shift" such that (total * w_max) >> shift <= u64::max
-        let shift = calculate_shift(total_bw, w_max as u64) as u8;
+        let shift = calculate_shift(total_bw, u64::from(w_max)) as u8;
 
         WeightSet {
             bandwidth_fn,
@@ -321,6 +324,23 @@ impl WeightSet {
             w,
         }
     }
+
+    /// Assert that we have correctly computed our shift values so that
+    /// our total weighted bws do not exceed u64::MAX.
+    fn validate(self, consensus: &MdConsensus) -> Self {
+        use WeightRole::*;
+        for role in [Guard, Middle, Exit, BeginDir, Unweighted] {
+            let _: u64 = consensus
+                .relays()
+                .iter()
+                .map(|rs| self.weight_rs_for_role(rs, role))
+                .fold(0_u64, |a, b| {
+                    a.checked_add(b)
+                        .expect("Incorrect relay weight calculation: total exceeded u64::MAX!")
+                });
+        }
+        self
+    }
 }
 
 /// The value to return if a weight parameter is absent.
@@ -367,7 +387,7 @@ fn calculate_shift(a: u64, b: u64) -> u32 {
 
 /// Return an upper bound for the log2 of n.
 ///
-/// This function overestimates whenver n is a power of two, but that doesn't
+/// This function overestimates whenever n is a power of two, but that doesn't
 /// much matter for the uses we're giving it here.
 fn log2_upper(n: u64) -> u32 {
     64 - n.leading_zeros()
@@ -375,8 +395,10 @@ fn log2_upper(n: u64) -> u32 {
 
 #[cfg(test)]
 mod test {
+    #![allow(clippy::unwrap_used)]
     use super::*;
     use netstatus::RelayWeight as RW;
+    use std::net::SocketAddr;
     use std::time::{Duration, SystemTime};
     use tor_netdoc::doc::netstatus::{Lifetime, RelayFlags, RouterStatusBuilder};
 
@@ -554,7 +576,7 @@ mod test {
         MdConsensus::builder()
             .rs()
             .identity([9; 20].into())
-            .add_or_port("127.0.0.1:9001".parse().unwrap())
+            .add_or_port(SocketAddr::from(([127, 0, 0, 1], 9001)))
             .doc_digest([9; 32])
             .protos("".parse().unwrap())
             .clone()
